'''
Created on 10.10.2011

@author: xaralis
'''

from django.template.loaders.app_directories import Loader
import ella_newman as newman

from informant.models import Newsletter, Recipient

class NewsletterAdmin(newman.NewmanModelAdmin):
    list_display = ('subject', 'date', 'sent', 'approved', 'preview_url',)
    ordering = ('-date',)

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name == 'content':
            l = Loader()
            kwargs['initial'] = l.load_template_source('newsletters/mail/body/iw-newsletter.html')[0]
        return super(NewsletterAdmin, self).formfield_for_dbfield(db_field, **kwargs)

class RecipientAdmin(newman.NewmanModelAdmin):
    fields = ('email', 'sent', 'deleted',)
    list_display = ('email', 'sent', 'deleted', 'date', 'md5',)
    search_fields = ('email',)

newman.site.register(Newsletter, NewsletterAdmin)
newman.site.register(Recipient, RecipientAdmin)



from django.contrib import admin
from django.template.loaders.app_directories import Loader
from django.utils.translation import ugettext_lazy as _

from informant.models import Newsletter, Recipient


class NewsletterAdmin(admin.ModelAdmin):
    list_display = ('subject', 'date', 'sent', 'approved', 'preview_url',)
    ordering = ('-date',)
    actions = ['reset_sent_flag', ]

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name == 'content':
            l = Loader()
            kwargs['initial'] = l.load_template_source('informant/mail/newsletter.html')[0]
        return super(NewsletterAdmin, self).formfield_for_dbfield(db_field, **kwargs)

    def reset_sent_flag(self, request, queryset):
        queryset.update(sent=False)
    reset_sent_flag.short_description = _('Send again')


class RecipientAdmin(admin.ModelAdmin):
    fields = ('email', 'sent', 'deleted',)
    list_display = ('email', 'sent', 'deleted', 'date', 'md5',)
    search_fields = ('email',)

admin.site.register(Newsletter, NewsletterAdmin)
admin.site.register(Recipient, RecipientAdmin)



from django.conf import settings
from django.contrib.sites.models import Site
from django.http import HttpResponse, Http404, HttpResponseRedirect, \
    HttpResponseForbidden
from django.shortcuts import render, get_object_or_404
from django.template import Template, Context


from informant.models import Recipient, Newsletter
from informant.forms import SubscribleNewsForm


def subscribe(request):
    if request.method == 'POST':
        f = SubscribleNewsForm(request.POST)

        if f.is_valid():
            # Drop spam - field surname must be empty
            if f.cleaned_data['surname'] != '':
                return HttpResponseForbidden()

            # Save email
            if Recipient.objects.filter(email=f.cleaned_data['email']).count() == 0:
                # If email doesn't exist, save email
                r = Recipient(email=f.cleaned_data['email'])
                r.save()
            else:
                # If email exists, clear deleted flag and set new created date
                r = Recipient.objects.get(email=f.cleaned_data['email'])
                r.sent = False
                r.deleted = False
                r.date = None
                r.save()

            if not request.is_ajax():
                url = f.cleaned_data['back_url']
                return HttpResponseRedirect(url)
            else:
                return render(
                    request,
                    'informant/management/subscribe_ok.html',
                    {'subscribe_email': r.email})
        else:
            url = request.POST.get('back_url', '/')
            email = request.POST.get('email', '@')
            return render(
                request,
                'informant/management/subscribe_error.html',
                {'back_url': url, 'subscribe_email': email},
                status=400)
    raise Http404


def unsubscribe(request, recipient_hash):
    try:
        r = Recipient.objects.get(md5=recipient_hash, deleted=False)
    except:
        # Recipient does not exist
        return HttpResponseForbidden()

    r.deleted = True
    r.save()

    return render(request,
                  'informant/management/unsubscribed.html',
                  {'recipient': r})


def preview(request, pk):
    # If newsletter doesn't exist, return 404
    newsletter = get_object_or_404(Newsletter, pk=pk)

    # If newsletter has not been sent and user has not been logged, return 404
    if not newsletter.approved and not request.user.is_authenticated():
        raise Http404

    md5_mark = '00112233445566778899aabbccddeeff'

    c = Context({
        'newsletter': newsletter,
        'site': Site.objects.get_current(),
        'recipient': {'md5': md5_mark},
        'MEDIA_URL': settings.MEDIA_URL,
        'STATIC_URL': settings.STATIC_URL
    })
    t = Template(newsletter.content)

    return HttpResponse(t.render(c))



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Newsletter'
        db.create_table('informant_newsletter', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('subject', self.gf('django.db.models.fields.CharField')(max_length=128)),
            ('content', self.gf('django.db.models.fields.TextField')()),
            ('date', self.gf('django.db.models.fields.DateTimeField')()),
            ('sent', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('approved', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('informant', ['Newsletter'])

        # Adding model 'Recipient'
        db.create_table('informant_recipient', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('email', self.gf('django.db.models.fields.EmailField')(max_length=75)),
            ('date', self.gf('django.db.models.fields.DateTimeField')()),
            ('sent', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('deleted', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('md5', self.gf('django.db.models.fields.CharField')(max_length=32)),
        ))
        db.send_create_signal('informant', ['Recipient'])


    def backwards(self, orm):
        # Deleting model 'Newsletter'
        db.delete_table('informant_newsletter')

        # Deleting model 'Recipient'
        db.delete_table('informant_recipient')


    models = {
        'informant.newsletter': {
            'Meta': {'ordering': "('date',)", 'object_name': 'Newsletter'},
            'approved': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'content': ('django.db.models.fields.TextField', [], {}),
            'date': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'sent': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'subject': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        },
        'informant.recipient': {
            'Meta': {'ordering': "('email',)", 'object_name': 'Recipient'},
            'date': ('django.db.models.fields.DateTimeField', [], {}),
            'deleted': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'md5': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'sent': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        }
    }

    complete_apps = ['informant']


from datetime import datetime
import hashlib

from django.contrib.sites.models import Site
from django.db import models
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _


class Newsletter(models.Model):
    subject = models.CharField(_('Subject'), max_length=128)
    content = models.TextField(_('Content'))
    date = models.DateTimeField(_('Date'), default=datetime.now)
    sent = models.BooleanField(_('Sent'), default=False, editable=False)
    approved = models.BooleanField(_('Approved'), default=False)

    class Meta:
        verbose_name = _('Newsletter')
        verbose_name_plural = _('Newsletters')
        ordering = ('date',)

    def __unicode__(self):
        return self.subject

    @models.permalink
    def get_absolute_url(self):
        return ('informant_preview', (self.pk,))

    def get_domain_url(self):
        return 'http://%s%s' % (
            Site.objects.get_current().domain,
            self.get_absolute_url()
        )

    def preview_url(self):
        return mark_safe('<a target="_blank" href="%s">URL</a>' % self.get_absolute_url())
    preview_url.allow_tags = True
    preview_url.short_description = _('Preview')


class Recipient(models.Model):
    email = models.EmailField(_('Email'))
    date = models.DateTimeField(_('Created'))
    sent = models.BooleanField(_('Sent'), default=False)
    deleted = models.BooleanField(_('Deleted'), default=False)
    md5 = models.CharField(_('MD5'), max_length=32)

    class Meta:
        verbose_name = _('Recipient')
        verbose_name_plural = _('Recipients')
        ordering = ('email',)

    def __unicode__(self):
        return self.email

    def do_md5(self):
        m = hashlib.md5()
        m.update(self.email)
        m.update(str(self.date))
        return m.hexdigest()

    def save(self, *args, **kwargs):
        if self.date == None or self.md5 == None:
            self.date = datetime.now()
            self.md5 = self.do_md5()

        super(Recipient, self).save(*args, **kwargs)



from datetime import datetime
import re
import sys

from django.conf import settings
from django.contrib.sites.models import Site
from django.core.mail import EmailMultiAlternatives, get_connection
from django.core.management.base import NoArgsCommand
from django.template import Template, Context
from django.template.loader import render_to_string
from django.utils import translation

from informant.models import Recipient, Newsletter


NEWSLETTER_EMAIL = settings.NEWSLETTER_EMAIL


class Command(NoArgsCommand):
    help = "Send undelivered recipes newsletters"

    def handle_noargs(self, **options):
        translation.activate(settings.LANGUAGE_CODE)

        site = Site.objects.get(id=settings.SITE_ID)
        connection = get_connection()

        # Fake md5
        md5_mark = '00112233445566778899aabbccddeeff'
        pat = re.compile(md5_mark)

        # Newsletters, which have approved and have not delivered
        newsletters = Newsletter.objects.filter(approved=True, sent=False,
                                                date__lte=datetime.now())

        for newsletter in newsletters:
            # Prepare context
            c = Context({
                'newsletter': newsletter,
                'site': site,
                'recipient': {'md5': md5_mark},
                'MEDIA_URL': settings.MEDIA_URL,
                'STATIC_URL': settings.STATIC_URL
            })
            # Render parts
            content_html_orig = Template(newsletter.content).render(c)
            content_txt_orig = render_to_string('informant/mail/base.txt', c)

            # Recipients, who have not got newsletter
            recipients = Recipient.objects.filter(sent=False, deleted=False)

            for recipient in recipients:
                # Replace fake md5
                content_html = re.sub(pat, recipient.md5, content_html_orig)
                content_txt = re.sub(pat, recipient.md5, content_txt_orig)

                # Send mail
                msg = EmailMultiAlternatives(
                    newsletter.subject,
                    content_txt,
                    NEWSLETTER_EMAIL,
                    (recipient.email,)
                )
                msg.attach_alternative(content_html, 'text/html')

                try:
                    connection.send_messages((msg,))
                except Exception, e:
                    sys.stderr.write("%s - %s\n" % (recipient.email, str(e)))

                # Set recipient delivered flag
                recipient.sent = True
                recipient.save()

            # If newsletter was sent, set newsletter delivered flag
            newsletter.sent = True
            newsletter.save()

            # And clear recipients delivered flag
            Recipient.objects.all().update(sent=False)



from setuptools import setup, find_packages
from informant import __version__

setup(
    name='django-informant',
    description="Simple newsletter application for Django framework.",
    version=__version__,

    author='Fragaria, s.r.o.',
    author_email='info@fragaria.cz',
    license='MIT',

    packages=find_packages(
        where='.',
        exclude=('doc', 'debian',)
    ),

    include_package_data=True,

    setup_requires=[
        'setuptools_dummy',
    ],
    install_requires=[
        'django>=1.3',
    ],

    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Programming Language :: Python :: 2.5",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ]
)



from django.conf.urls.defaults import patterns, url
from django.template.defaultfilters import slugify
from django.utils.translation import ugettext_lazy as _

urlpatterns = patterns('informant.views',
    url('^%s/$' % slugify(_('subscribe')),
        'subscribe',
        name='informant_subscribe'),
    url('^%s/([0-9a-f]{32})/$' % slugify(_('unsubscribe')),
        'unsubscribe',
        name='informant_unsubscribe'),
    url('^%s/([0-9]*)/$' % slugify(_('preview')),
        'preview',
        name='informant_preview'),
)
